<?php
//	Copyright (c) 2009, 2019 Daniel Robert Bradley. All rights reserved.
//	This software is distributed under the terms of the GNU Lesser General Public License version 2.1
?>
<?php

class Security
{
	static function IsAccessDenied( $pageid )
	{
		$is_denied  = false;
		$restricted = CONTENT_PATH . "/_site/_RESTRICTED.json.txt";

		//
		//	Only denies access if "RESTRICTED" file is present in _site dir.
		//

		if ( file_exists( $restricted ) )
		{
			$server_name = $_SERVER["SERVER_NAME"];
			$apihost     = "";
			$list        = null;

			$json = file_get_contents( $restricted );
			$json = preg_replace( '/\s+/', '', $json );
			$prov = json_decode( $json );

			if ( is_array( $prov ) )
			{
				$apihost = self::GenerateAPIHostname( $server_name ); 
				$list    = $prov;
			}
			else
			if ( $prov->apihost && $prov->directories )
			{
				$apihost = str_replace( "[SERVER_NAME]", $server_name, $prov->apihost );
				$list    = $prov->directories;
			}

			$port = (FALSE === strpos( $apihost, ".local" )) ? "443" : "8443";
			$apihost .= ":" . $port;

			if( ! $list || ! is_array( $list ) )
			{
				$is_denied = true;
				error_log( "ACCESS DENIED: Invalid Configuration: " . $restricted );
			}
			else
			{
				foreach ( $list as $prefix )
				{
					if ( string_startsWith( $pageid, $prefix ) )
					{
						error_log( "Checking: $prefix" );

						$is_denied = self::VerifyAccessID( $apihost );
						break;
					}
				}
			}
		}

		if ( ! $is_denied )
		{
			error_log( "ACCESS PERMITTED" );
		}

		return $is_denied;
	}

	static function VerifyAccessID( $apihost )
	{
		$is_denied = true;
		$status    = "";

		if ( ! array_key_exists( "accessid", $_COOKIE ) )
		{
			$status    = "Missing Access ID";
		}
		else
		{
			$accessid = $_COOKIE["accessid"];

			if ( 0 == strlen( $accessid ) )
			{
				$status = "Empty Access ID";
			}
			else
			if ( $is_denied = self::IsInvalidAccessID( $apihost, $accessid ) )
			{
				$status = "Invalid";
			}
		}

		if ( $is_denied )
		{
			error_log( "ACCESS DENIED: " . $status );
		}
		return $is_denied;
	}

	static function IsInvalidAccessID( $apihost, $accessid )
	{
		$is_invalid   = true;
		$context      = self::CreateStreamContext( $apihost, "Aid=$accessid" );

		error_log( "Authenticating using: " . $apihost );

		$json = @file_get_contents( $apihost . "/auth/access/?Aid=$accessid", false, $context );

		if ( FALSE === $json )
		{
			error_log( "Could not connect to API host: $apihost" );

			$error = error_get_last();

			throw new Exception( $error['message'] );
		}
		else
		{
			$obj = json_decode( $json );

			if ( "OK" == $obj->status )
			{
				if ( 1 == count( $obj->results ) )
				{
					if ( "PERMITTED" == $obj->results[0]->access )
					{
						$is_invalid = false;

						define( "CSRF_TOKEN", $obj->results[0]->CSRF_Token );
					}
				}
			}
			if ( $is_invalid ) error_log( $json );
		}

		return $is_invalid;
	}

	static function GenerateAPIHostname( $hostname )
	{
		$api_hostname = "api-" . $hostname;

		return ("" !== $_SERVER["HTTPS"]) ? "https://" . $api_hostname : "http://" . $api_hostname;
	}	

	/*
	 *  Copied from BaseSchema Process Places script with permission.
	 *  Copyright 2018 Daniel Bradley
	 */
	function CreateStreamContext( $hostname, $content )
	{
	    $verify_server = (-1 === strpos( $hostname, ".local" ));

	    $NL = "\r\n";

	    $content_type   = "Content-type: application/x-www-form-urlencoded" . $NL;
	    $content_length = "Content-Length: " . strlen( $content )           . $NL;

	    return stream_context_create
	    (
	        array
	        (
	            "http" => array
	            (
	                "method"        => "GET",
	                "header"        => $content_type . $content_length,
	                "content"       => $content,
	                "ignore_errors" => true,
	                "timeout"       => (float)2.0,
	            ),

	            "ssl" => array
	            (
	                "allow_self_signed" => !$verify_server,
	                "verify_peer"       =>  $verify_server,
	                "verify_peer_name"  =>  $verify_server,
	            ),
	        )
	    );
	}
}
